description = [
    {
        "description": "Find all Open Reading Frames (ORFs) in a DNA sequence using "
        "Biopython, searching both forward and reverse complement "
        "strands.",
        "name": "annotate_open_reading_frames",
        "optional_parameters": [
            {
                "default": False,
                "description": "Whether to search the reverse complement strand",
                "name": "search_reverse",
                "type": "bool",
            },
            {
                "default": False,
                "description": "Whether to filter out ORFs with same end but later start",
                "name": "filter_subsets",
                "type": "bool",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "DNA sequence to analyze",
                "name": "sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Minimum length of ORF in nucleotides",
                "name": "min_length",
                "type": "int",
            },
        ],
    },
    {
        "description": "Annotate a DNA sequence using pLannotate's command-line interface.",
        "name": "annotate_plasmid",
        "optional_parameters": [
            {
                "default": True,
                "description": "Whether the sequence is circular",
                "name": "is_circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The DNA sequence to annotate",
                "name": "sequence",
                "type": "str",
            }
        ],
    },
    {
        "description": "Retrieves the coding sequence(s) of a specified gene from NCBI Entrez.",
        "name": "get_gene_coding_sequence",
        "optional_parameters": [
            {
                "default": None,
                "description": "Email address for NCBI Entrez (recommended)",
                "name": "email",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Name of the gene",
                "name": "gene_name",
                "type": "str",
            },
            {
                "default": None,
                "description": "Name of the organism",
                "name": "organism",
                "type": "str",
            },
        ],
    },
    {
        "description": "Unified function to retrieve plasmid sequences from either "
        "Addgene or NCBI. If is_addgene is True or identifier is "
        "numeric, uses Addgene. Otherwise searches NCBI using the "
        "plasmid name.",
        "name": "get_plasmid_sequence",
        "optional_parameters": [
            {
                "default": None,
                "description": "Force Addgene lookup if True, force "
                "NCBI if False. If None, attempts to "
                "auto-detect based on identifier "
                "format.",
                "name": "is_addgene",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Either an Addgene ID or plasmid name",
                "name": "identifier",
                "type": "str",
            }
        ],
    },
    {
        "description": "Align short sequences (primers) to a longer sequence, "
        "allowing for one mismatch. Checks both forward and reverse "
        "complement strands.",
        "name": "align_sequences",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "Target DNA sequence",
                "name": "long_seq",
                "type": "str",
            },
            {
                "default": None,
                "description": "Single primer or list of primers",
                "name": "short_seqs",
                "type": "Union[str, List[str]]",
            },
        ],
    },
    {
        "description": "Simulate PCR amplification with given primers and sequence.",
        "name": "pcr_simple",
        "optional_parameters": [
            {
                "default": False,
                "description": "Whether the sequence is circular",
                "name": "circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Either a sequence string or path to plasmid file",
                "name": "sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Forward primer sequence (5' to 3')",
                "name": "forward_primer",
                "type": "str",
            },
            {
                "default": None,
                "description": "Reverse primer sequence (5' to 3')",
                "name": "reverse_primer",
                "type": "str",
            },
        ],
    },
    {
        "description": "Simulates restriction enzyme digestion of a DNA sequence and "
        "returns the resulting fragments with their properties.",
        "name": "digest_sequence",
        "optional_parameters": [
            {
                "default": True,
                "description": "Whether the DNA sequence is circular (True) or linear (False)",
                "name": "is_circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Input DNA sequence to be digested",
                "name": "dna_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Names of restriction enzymes to use for digestion",
                "name": "enzyme_names",
                "type": "List[str]",
            },
        ],
    },
    {
        "description": "Identifies restriction enzyme sites in a given DNA sequence for specified enzymes.",
        "name": "find_restriction_sites",
        "optional_parameters": [
            {
                "default": True,
                "description": "Whether the DNA sequence is circular (True) or linear (False)",
                "name": "is_circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Complete input DNA sequence",
                "name": "dna_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "List of restriction enzyme names to check",
                "name": "enzymes",
                "type": "List[str]",
            },
        ],
    },
    {
        "description": "Finds common restriction enzyme sites in a DNA sequence and returns their cut positions.",
        "name": "find_restriction_enzymes",
        "optional_parameters": [
            {
                "default": False,
                "description": "Whether the sequence is circular",
                "name": "is_circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "DNA sequence to analyze",
                "name": "sequence",
                "type": "str",
            }
        ],
    },
    {
        "description": "Compare query sequence against reference sequence to identify mutations.",
        "name": "find_sequence_mutations",
        "optional_parameters": [
            {
                "default": 1,
                "description": "The start position of the query sequence",
                "name": "query_start",
                "type": "int",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The sequence being analyzed",
                "name": "query_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "The reference sequence to compare against",
                "name": "reference_sequence",
                "type": "str",
            },
        ],
    },
    {
        "description": "Design sgRNAs for CRISPR knockout by searching pre-computed "
        "sgRNA libraries. Returns optimized guide RNAs for targeting "
        "a specific gene.",
        "name": "design_knockout_sgrna",
        "optional_parameters": [
            {
                "default": "human",
                "description": "Target organism species",
                "name": "species",
                "type": "str",
            },
            {
                "default": 1,
                "description": "Number of guides to return",
                "name": "num_guides",
                "type": "int",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": 'Target gene symbol/name (e.g., "EGFR", "TP53")',
                "name": "gene_name",
                "type": "str",
            },
            {
                "default": None,
                "description": "Path to the data lake",
                "name": "data_lake_path",
                "type": "str",
            },
        ],
    },
    {
        "description": "Return a standard protocol for annealing oligonucleotides without phosphorylation.",
        "name": "get_oligo_annealing_protocol",
        "optional_parameters": [],
        "required_parameters": [],
    },
    {
        "description": "Return a customized protocol for Golden Gate assembly based "
        "on the number of inserts and specific DNA sequences.",
        "name": "get_golden_gate_assembly_protocol",
        "optional_parameters": [
            {
                "default": 1,
                "description": "Number of inserts to be assembled",
                "name": "num_inserts",
                "type": "int",
            },
            {
                "default": 75.0,
                "description": "Amount of vector to use in ng",
                "name": "vector_amount_ng",
                "type": "float",
            },
            {
                "default": None,
                "description": "List of insert lengths in bp",
                "name": "insert_lengths",
                "type": "List[int]",
            },
            {
                "default": False,
                "description": "Whether this is for library preparation",
                "name": "is_library_prep",
                "type": "bool",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Type IIS restriction enzyme to be used",
                "name": "enzyme_name",
                "type": "str",
            },
            {
                "default": None,
                "description": "Length of the destination vector in bp",
                "name": "vector_length",
                "type": "int",
            },
        ],
    },
    {
        "description": "Return a standard protocol for bacterial transformation.",
        "name": "get_bacterial_transformation_protocol",
        "optional_parameters": [
            {
                "default": "ampicillin",
                "description": "Selection antibiotic",
                "name": "antibiotic",
                "type": "str",
            },
            {
                "default": False,
                "description": "Whether the sequence contains repetitive elements",
                "name": "is_repetitive",
                "type": "bool",
            },
        ],
        "required_parameters": [],
    },
    {
        "description": "Design a single primer within the given sequence window.",
        "name": "design_primer",
        "optional_parameters": [
            {
                "default": 20,
                "description": "Length of the primer to design",
                "name": "primer_length",
                "type": "int",
            },
            {
                "default": 0.4,
                "description": "Minimum GC content",
                "name": "min_gc",
                "type": "float",
            },
            {
                "default": 0.6,
                "description": "Maximum GC content",
                "name": "max_gc",
                "type": "float",
            },
            {
                "default": 55.0,
                "description": "Minimum melting temperature in °C",
                "name": "min_tm",
                "type": "float",
            },
            {
                "default": 65.0,
                "description": "Maximum melting temperature in °C",
                "name": "max_tm",
                "type": "float",
            },
            {
                "default": 100,
                "description": "Size of window to search for primers",
                "name": "search_window",
                "type": "int",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Target DNA sequence",
                "name": "sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Starting position for primer search",
                "name": "start_pos",
                "type": "int",
            },
        ],
    },
    {
        "description": "Design Sanger sequencing primers to verify a specific region "
        "in a plasmid. First tries to use primers from an existing "
        "primer pool. If they cannot fully cover the region, designs "
        "additional primers as needed.",
        "name": "design_verification_primers",
        "optional_parameters": [
            {
                "default": None,
                "description": "List of existing primers with their sequences and optional names",
                "name": "existing_primers",
                "type": "Optional[List[Dict[str, str]]]",
            },
            {
                "default": True,
                "description": "Whether the plasmid is circular",
                "name": "is_circular",
                "type": "bool",
            },
            {
                "default": 800,
                "description": "Typical read length for each primer in base pairs",
                "name": "coverage_length",
                "type": "int",
            },
            {
                "default": 20,
                "description": "Length of newly designed primers",
                "name": "primer_length",
                "type": "int",
            },
            {
                "default": 0.4,
                "description": "Minimum GC content for new primers",
                "name": "min_gc",
                "type": "float",
            },
            {
                "default": 0.6,
                "description": "Maximum GC content for new primers",
                "name": "max_gc",
                "type": "float",
            },
            {
                "default": 55.0,
                "description": "Minimum melting temperature in °C",
                "name": "min_tm",
                "type": "float",
            },
            {
                "default": 65.0,
                "description": "Maximum melting temperature in °C",
                "name": "max_tm",
                "type": "float",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The complete plasmid sequence",
                "name": "plasmid_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Start and end positions to verify (0-based indexing)",
                "name": "target_region",
                "type": "Tuple[int, int]",
            },
        ],
    },
    {
        "description": "Design complementary oligonucleotides with Type IIS "
        "restriction enzyme overhangs for Golden Gate assembly based "
        "on restriction site analysis of the backbone.",
        "name": "design_golden_gate_oligos",
        "optional_parameters": [
            {
                "default": True,
                "description": "Whether the backbone is circular",
                "name": "is_circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Complete backbone sequence",
                "name": "backbone_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Sequence to be inserted (e.g., sgRNA target sequence)",
                "name": "insert_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": "Type IIS restriction enzyme to be used",
                "name": "enzyme_name",
                "type": "str",
            },
        ],
    },
    {
        "description": "Simulate Golden Gate assembly to predict final construct "
        "sequences from backbone and fragment sequences.",
        "name": "golden_gate_assembly",
        "optional_parameters": [
            {
                "default": True,
                "description": "Whether the backbone is circular",
                "name": "is_circular",
                "type": "bool",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Complete backbone sequence",
                "name": "backbone_sequence",
                "type": "str",
            },
            {
                "default": None,
                "description": 'Type IIS restriction enzyme to be used (e.g., "BsmBI", "BsaI")',
                "name": "enzyme_name",
                "type": "str",
            },
            {
                "default": None,
                "description": "List of fragments to insert, containing one of: "
                "name + fwd_oligo + rev_oligo (oligo pair with matching overhangs) or "
                "name + sequence (double-stranded DNA fragment containing restriction sites)",
                "name": "fragments",
                "type": "List[Dict[str, str]]",
            },
        ],
    },
]



description = [
    {
        "description": "Fetches supplementary information for a paper given its DOI "
        "and saves it to a specified directory.",
        "name": "fetch_supplementary_info_from_doi",
        "optional_parameters": [
            {
                "default": "supplementary_info",
                "description": "Directory to save supplementary files",
                "name": "output_dir",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The paper DOI",
                "name": "doi",
                "type": "str",
            }
        ],
    },
    {
        "description": "Query arXiv for papers based on the provided search query.",
        "name": "query_arxiv",
        "optional_parameters": [
            {
                "default": 10,
                "description": "The maximum number of papers to retrieve.",
                "name": "max_papers",
                "type": "int",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The search query string.",
                "name": "query",
                "type": "str",
            }
        ],
    },
    {
        "description": "Query Google Scholar for papers based on the provided search "
        "query and return the first search result.",
        "name": "query_scholar",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "The search query string.",
                "name": "query",
                "type": "str",
            }
        ],
    },
    {
        "description": "Query PubMed for papers based on the provided search query.",
        "name": "query_pubmed",
        "optional_parameters": [
            {
                "default": 10,
                "description": "The maximum number of papers to retrieve.",
                "name": "max_papers",
                "type": "int",
            },
            {
                "default": 3,
                "description": "Maximum number of retry attempts with modified queries.",
                "name": "max_retries",
                "type": "int",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The search query string.",
                "name": "query",
                "type": "str",
            }
        ],
    },
    {
        "description": "Search using Google search and return formatted results.",
        "name": "search_google",
        "optional_parameters": [
            {
                "default": 3,
                "description": "Number of results to return",
                "name": "num_results",
                "type": "int",
            },
            {
                "default": "en",
                "description": "Language code for search results",
                "name": "language",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": 'The search query (e.g., "protocol text or search question")',
                "name": "query",
                "type": "str",
            }
        ],
    },
    {
        "description": "Extract the text content of a webpage using requests and BeautifulSoup.",
        "name": "extract_url_content",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "Webpage URL to extract content from",
                "name": "url",
                "type": "str",
            }
        ],
    },
    {
        "description": "Extract text content from a PDF file.",
        "name": "extract_pdf_content",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "URL of the PDF file",
                "name": "url",
                "type": "str",
            }
        ],
    },
]



description = [
    {
        "description": "Predicts intrinsically disordered regions (IDRs) in a protein sequence using IUPred2A.",
        "name": "predict_protein_disorder_regions",
        "optional_parameters": [
            {
                "default": 0.5,
                "description": "The disorder score threshold above which a residue is considered disordered",
                "name": "threshold",
                "type": "float",
            },
            {
                "default": "disorder_prediction_results.csv",
                "description": "Filename to save the per-residue disorder scores",
                "name": "output_file",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The amino acid sequence of the protein to analyze",
                "name": "protein_sequence",
                "type": "str",
            }
        ],
    },
    {
        "description": "Quantifies cell morphology and cytoskeletal organization from fluorescence microscopy images.",
        "name": "analyze_cell_morphology_and_cytoskeleton",
        "optional_parameters": [
            {
                "default": "./results",
                "description": "Directory to save output files",
                "name": "output_dir",
                "type": "str",
            },
            {
                "default": "otsu",
                "description": "Method for cell segmentation ('otsu', 'adaptive', or 'manual')",
                "name": "threshold_method",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to the fluorescence microscopy image file",
                "name": "image_path",
                "type": "str",
            }
        ],
    },
    {
        "description": "Quantify tissue deformation and flow dynamics from microscopy image sequence.",
        "name": "analyze_tissue_deformation_flow",
        "optional_parameters": [
            {
                "default": "results",
                "description": "Directory to save results",
                "name": "output_dir",
                "type": "str",
            },
            {
                "default": 1.0,
                "description": "Physical scale of pixels (e.g., μm/pixel) for proper scaling of metrics",
                "name": "pixel_scale",
                "type": "float",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Sequence of microscopy images "
                "(either a list of file paths or a "
                "3D numpy array [time, height, "
                "width])",
                "name": "image_sequence",
                "type": "list or numpy.ndarray",
            }
        ],
    },
]



description = [
    {
        "description": "Perform Flux Balance Analysis (FBA) on a genome-scale "
        "metabolic network model and return a research log of the "
        "process and results.",
        "name": "perform_flux_balance_analysis",
        "optional_parameters": [
            {
                "default": None,
                "description": "Dictionary of reaction constraints "
                "where keys are reaction IDs and "
                "values are tuples of (lower_bound, "
                "upper_bound)",
                "name": "constraints",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Reaction ID to use as the objective function (e.g., biomass reaction)",
                "name": "objective_reaction",
                "type": "str",
            },
            {
                "default": "fba_results.csv",
                "description": "File name to save the flux distribution results",
                "name": "output_file",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to the metabolic model file (SBML or JSON format)",
                "name": "model_file",
                "type": "str",
            }
        ],
    },
    {
        "description": "Model protein dimerization networks to find equilibrium concentrations of dimers.",
        "name": "model_protein_dimerization_network",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "Dictionary mapping monomer names to their initial concentrations (in arbitrary units)",
                "name": "monomer_concentrations",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Dictionary mapping dimer names (as 'A-B' strings) to their association constants (Ka)",
                "name": "dimerization_affinities",
                "type": "dict",
            },
            {
                "default": None,
                "description": "List of (monomer1, monomer2) pairs that can form dimers",
                "name": "network_topology",
                "type": "list",
            },
        ],
    },
    {
        "description": "Construct and simulate kinetic models of metabolic networks "
        "and analyze their responses to perturbations.",
        "name": "simulate_metabolic_network_perturbation",
        "optional_parameters": [
            {"default": 100, "description": "Total simulation time", "name": "simulation_time", "type": "float"},
            {"default": 1000, "description": "Number of time points to simulate", "name": "time_points", "type": "int"},
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to the COBRA model file (SBML format)",
                "name": "model_file",
                "type": "str",
            },
            {
                "default": None,
                "description": "Dictionary mapping metabolite IDs to their initial concentrations",
                "name": "initial_concentrations",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Dictionary with keys 'time' "
                "(float), 'metabolite' (str), and "
                "'factor' (float) for perturbation "
                "details",
                "name": "perturbation_params",
                "type": "dict",
            },
        ],
    },
    {
        "description": "Simulate protein signaling network dynamics using ODE-based "
        "logic modeling with normalized Hill functions.",
        "name": "simulate_protein_signaling_network",
        "optional_parameters": [
            {
                "default": 100,
                "description": "Total simulation time in arbitrary time units.",
                "name": "simulation_time",
                "type": "float",
            },
            {
                "default": 1000,
                "description": "Number of time points for the simulation.",
                "name": "time_points",
                "type": "int",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Dictionary defining the network "
                "topology. Each key is a target "
                "protein and its value is a list of "
                "tuples (regulator, regulation_type) "
                "where regulation_type is 1 for "
                "activation and -1 for inhibition.",
                "name": "network_structure",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Dictionary of reaction parameters. "
                "Keys are tuples (regulator, target) "
                "and values are dictionaries with "
                "keys 'W' (weight), 'n' (Hill "
                "coefficient), and 'EC50' "
                "(half-maximal effective "
                "concentration).",
                "name": "reaction_params",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Dictionary of species parameters. "
                "Keys are protein names and values "
                "are dictionaries with keys 'tau' "
                "(time constant), 'y0' (initial "
                "concentration), and 'ymax' (maximum "
                "concentration).",
                "name": "species_params",
                "type": "dict",
            },
        ],
    },
    {
        "description": "Compares two protein structures to identify structural differences and conformational changes.",
        "name": "compare_protein_structures",
        "optional_parameters": [
            {
                "default": "A",
                "description": "Chain ID to analyze in the first structure",
                "name": "chain_id1",
                "type": "str",
            },
            {
                "default": "A",
                "description": "Chain ID to analyze in the second structure",
                "name": "chain_id2",
                "type": "str",
            },
            {
                "default": "protein_comparison",
                "description": "Prefix for output files",
                "name": "output_prefix",
                "type": "str",
            },
        ],
        "required_parameters": [
            {"default": None, "description": "Path to the first PDB file", "name": "pdb_file1", "type": "str"},
            {"default": None, "description": "Path to the second PDB file", "name": "pdb_file2", "type": "str"},
        ],
    },
    {
        "description": "Simulate the time-dependent concentrations of renin-angiotensin system (RAS) components.",
        "name": "simulate_renin_angiotensin_system_dynamics",
        "optional_parameters": [
            {
                "default": 48,
                "description": "Total simulation time in hours",
                "name": "simulation_time",
                "type": "float",
            },
            {"default": 100, "description": "Number of time points to evaluate", "name": "time_points", "type": "int"},
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Initial concentrations of RAS "
                "components with keys: 'renin', "
                "'angiotensinogen', 'angiotensin_I', "
                "'angiotensin_II', "
                "'ACE2_angiotensin_II', "
                "'angiotensin_1_7'",
                "name": "initial_concentrations",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Kinetic rate constants with keys: "
                "'k_ren', 'k_agt', 'k_ace', "
                "'k_ace2', 'k_at1r', 'k_mas'",
                "name": "rate_constants",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Parameters controlling feedback mechanisms with keys: 'fb_ang_II', 'fb_ace2'",
                "name": "feedback_params",
                "type": "dict",
            },
        ],
    },
    {
        "description": "Answer functions and properties questions for DNA sequences ",
        "name": "query_chatnt",
        "optional_parameters": [
            {
                "default": -1,
                "description": "Device to use for the ChatNT model. Default is -1 (CPU).",
                "name": "device",
                "type": "int",
            }
        ],
        "required_parameters": [
            {"default": "A", "description": "Questions about the DNA sequence", "name": "question", "type": "str"},
            {"default": "A", "description": "DNA sequence with potential functions", "name": "sequence", "type": "str"},
        ],
    },
]



description = [
    {
        "description": "Analyzes circular dichroism (CD) spectroscopy data to "
        "determine secondary structure and thermal stability.",
        "name": "analyze_circular_dichroism_spectra",
        "optional_parameters": [
            {
                "default": None,
                "description": "Temperature values (°C) for thermal denaturation experiment",
                "name": "temperature_data",
                "type": "list or numpy.ndarray",
            },
            {
                "default": None,
                "description": "CD signal values at specific wavelength across different temperatures",
                "name": "thermal_cd_data",
                "type": "list or numpy.ndarray",
            },
            {
                "default": "./",
                "description": "Directory to save result files",
                "name": "output_dir",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": 'Name of the biomolecule sample (e.g., "Znf706", "G-quadruplex")',
                "name": "sample_name",
                "type": "str",
            },
            {
                "default": None,
                "description": 'Type of biomolecule ("protein" or "nucleic_acid")',
                "name": "sample_type",
                "type": "str",
            },
            {
                "default": None,
                "description": "Wavelength values in nm for CD spectrum",
                "name": "wavelength_data",
                "type": "list or numpy.ndarray",
            },
            {
                "default": None,
                "description": "CD signal intensity values (typically in mdeg or Δε)",
                "name": "cd_signal_data",
                "type": "list or numpy.ndarray",
            },
        ],
    },
    {
        "description": "Calculate numeric values for various structural features of an RNA secondary structure.",
        "name": "analyze_rna_secondary_structure_features",
        "optional_parameters": [
            {
                "default": None,
                "description": "The RNA sequence corresponding to "
                "the structure. If provided, "
                "sequence-dependent energy "
                "calculations will be performed.",
                "name": "sequence",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "RNA secondary structure in "
                "dot-bracket notation (e.g., "
                '"(((...)))"). Parentheses represent '
                "base pairs, dots represent unpaired "
                "bases.",
                "name": "dot_bracket_structure",
                "type": "str",
            }
        ],
    },
    {
        "description": "Analyze protease kinetics data from fluorogenic peptide "
        "cleavage assays, fit the data to Michaelis-Menten kinetics, "
        "and determine key kinetic parameters.",
        "name": "analyze_protease_kinetics",
        "optional_parameters": [
            {
                "default": "protease_kinetics",
                "description": "Prefix for output files",
                "name": "output_prefix",
                "type": "str",
            },
            {
                "default": "./",
                "description": "Directory to save output files",
                "name": "output_dir",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Array of time points (in seconds) at which measurements were taken",
                "name": "time_points",
                "type": "numpy.ndarray",
            },
            {
                "default": None,
                "description": "2D array of fluorescence "
                "measurements where each row "
                "corresponds to a different "
                "substrate concentration and each "
                "column corresponds to a time point",
                "name": "fluorescence_data",
                "type": "numpy.ndarray",
            },
            {
                "default": None,
                "description": "Array of substrate concentrations "
                "(in μM) corresponding to each row "
                "in fluorescence_data",
                "name": "substrate_concentrations",
                "type": "numpy.ndarray",
            },
            {
                "default": None,
                "description": "Concentration of the protease enzyme (in μM)",
                "name": "enzyme_concentration",
                "type": "float",
            },
        ],
    },
    {
        "description": "Performs in vitro enzyme kinetics assay and analyzes the dose-dependent effects of modulators.",
        "name": "analyze_enzyme_kinetics_assay",
        "optional_parameters": [
            {
                "default": None,
                "description": "Dictionary of modulators where keys "
                "are modulator names and values are "
                "lists of concentrations in μM",
                "name": "modulators",
                "type": "dict",
            },
            {
                "default": None,
                "description": "Time points in minutes for time-course measurements",
                "name": "time_points",
                "type": "list or numpy.ndarray",
            },
            {
                "default": "./",
                "description": "Directory to save output files",
                "name": "output_dir",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Name of the purified enzyme being tested",
                "name": "enzyme_name",
                "type": "str",
            },
            {
                "default": None,
                "description": "List of substrate concentrations in μM for kinetic analysis",
                "name": "substrate_concentrations",
                "type": "list or numpy.ndarray",
            },
            {
                "default": None,
                "description": "Concentration of the enzyme in nM",
                "name": "enzyme_concentration",
                "type": "float",
            },
        ],
    },
    {
        "description": "Analyzes isothermal titration calorimetry (ITC) data to "
        "determine binding affinity and thermodynamic parameters.",
        "name": "analyze_itc_binding_thermodynamics",
        "optional_parameters": [
            {
                "default": None,
                "description": "Path to CSV or TSV file containing "
                "ITC thermogram data with columns "
                "for injection number, injection "
                "volume, and heat released/absorbed.",
                "name": "itc_data_path",
                "type": "str",
            },
            {
                "default": None,
                "description": "Raw ITC thermogram data as a numpy "
                "array with shape (n_injections, 3) "
                "containing injection number, "
                "injection volume, and heat.",
                "name": "itc_data",
                "type": "numpy.ndarray",
            },
            {
                "default": 298.15,
                "description": "Temperature in Kelvin at which the experiment was conducted.",
                "name": "temperature",
                "type": "float",
            },
            {
                "default": None,
                "description": "Initial concentration of protein in the cell in molar (M).",
                "name": "protein_concentration",
                "type": "float",
            },
            {
                "default": None,
                "description": "Concentration of ligand in the syringe in molar (M).",
                "name": "ligand_concentration",
                "type": "float",
            },
        ],
        "required_parameters": [],
    },
    {
        "description": "Perform multiple sequence alignment and phylogenetic "
        "analysis to identify conserved protein regions.",
        "name": "analyze_protein_conservation",
        "optional_parameters": [
            {
                "default": "./",
                "description": "Directory to save output files",
                "name": "output_dir",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "List of protein sequences in FASTA format from multiple organisms.",
                "name": "protein_sequences",
                "type": "list of str",
            }
        ],
    },
]



description = [
    {
        "description": "Annotate cell types based on gene markers and transferred "
        "labels using LLM. After leiden clustering, annotate clusters "
        "using differentially expressed genes and optionally "
        "incorporate transferred labels from reference datasets.",
        "name": "annotate_celltype_scRNA",
        "optional_parameters": [
            {
                "default": "leiden",
                "description": "Clustering method to use for cell type annotation",
                "name": "cluster",
                "type": "str",
            },
            {
                "default": "claude-3-5-sonnet-20241022",
                "description": "Language model instance for cell type prediction",
                "name": "llm",
                "type": "str",
            },
            {
                "default": None,
                "description": "Transferred cell type composition for each cluster",
                "name": "composition",
                "type": "pd.DataFrame",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Name of the AnnData file containing scRNA-seq data",
                "name": "adata_filename",
                "type": "str",
            },
            {
                "default": None,
                "description": "Directory containing the data files",
                "name": "data_dir",
                "type": "str",
            },
            {
                "default": None,
                "description": 'Information about the scRNA-seq data (e.g., "homo sapiens, brain tissue, normal")',
                "name": "data_info",
                "type": "str",
            },
            {
                "default": None,
                "description": "Path to the data lake",
                "name": "data_lake_path",
                "type": "str",
            },
        ],
    },
    {
        "description": "Perform cell type annotation of single-cell RNA-seq data using Panhuman Azimuth Neural Network. "
        "This function implements the Panhuman Azimuth workflow for cell type annotation using the "
        "panhumanpy package, providing hierarchical cell type labels for tissues across the human body. ",
        "name": "annotate_celltype_with_panhumanpy",
        "optional_parameters": [
            {
                "default": None,
                "description": "Column name in adata.var containing gene symbols (default: None, uses index)",
                "name": "feature_names_col",
                "type": "str",
            },
            {
                "default": True,
                "description": "Whether to perform additional label refinement for consistent granularity",
                "name": "refine",
                "type": "bool",
            },
            {
                "default": True,
                "description": "Whether to generate ANN embeddings and UMAP",
                "name": "umap",
                "type": "bool",
            },
            {
                "default": "./output",
                "description": "Directory to save results",
                "name": "output_dir",
                "type": "str",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to the AnnData file containing scRNA-seq data",
                "name": "adata_path",
                "type": "str",
            },
        ],
    },
    {
        "description": "Create scVI and scANVI embeddings for single-cell RNA-seq "
        "data, saving the results to an AnnData object.",
        "name": "create_scvi_embeddings_scRNA",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "Filename of the AnnData object to load",
                "name": "adata_filename",
                "type": "str",
            },
            {
                "default": None,
                "description": "Column name in adata.obs for batch information",
                "name": "batch_key",
                "type": "str",
            },
            {
                "default": None,
                "description": "Column name in adata.obs for cell type labels",
                "name": "label_key",
                "type": "str",
            },
            {
                "default": None,
                "description": "Directory path where the AnnData file is located and where output will be saved",
                "name": "data_dir",
                "type": "str",
            },
        ],
    },
    {
        "description": "Performs batch integration on single-cell RNA-seq data using "
        "Harmony and saves the integrated embeddings.",
        "name": "create_harmony_embeddings_scRNA",
        "optional_parameters": [],
        "required_parameters": [
            {
                "default": None,
                "description": "Filename of the AnnData object to load",
                "name": "adata_filename",
                "type": "str",
            },
            {
                "default": None,
                "description": "Column name in adata.obs that defines the batch variable for integration",
                "name": "batch_key",
                "type": "str",
            },
            {
                "default": None,
                "description": "Directory path where the input file is located and output will be saved",
                "name": "data_dir",
                "type": "str",
            },
        ],
    },
    {
        "description": "Generate UCE embeddings for single-cell RNA-seq data and map "
        "them to a reference dataset for cell type annotation.",
        "name": "get_uce_embeddings_scRNA",
        "optional_parameters": [
            {
                "default": "/dfs/project/bioagentos/data/singlecell/",
                "description": "Root directory for single-cell data storage",
                "name": "DATA_ROOT",
                "type": "str",
            },
            {
                "default": None,
                "description": "Custom command line arguments to pass to the UCE script",
                "name": "custom_args",
                "type": "List[str]",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Filename of the AnnData object to process",
                "name": "adata_filename",
                "type": "str",
            },
            {
                "default": None,
                "description": "Directory where the input data is stored and output will be saved",
                "name": "data_dir",
                "type": "str",
            },
        ],
    },
    {
        "description": "Map cell embeddings from the input dataset to the Integrated "
        "Megascale Atlas reference dataset using UCE embeddings.",
        "name": "map_to_ima_interpret_scRNA",
        "optional_parameters": [
            {
                "default": None,
                "description": "Dictionary of custom arguments "
                "including 'n_neighbors' and "
                "'metric' for nearest neighbor "
                "search",
                "name": "custom_args",
                "type": "dict",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Filename of the AnnData object to be mapped",
                "name": "adata_filename",
                "type": "str",
            },
            {
                "default": None,
                "description": "Directory containing the AnnData file",
                "name": "data_dir",
                "type": "str",
            },
        ],
    },
    {
        "description": "Given a gene name, fetch RNA-seq expression data showing the "
        "top K tissues with highest transcripts-per-million (TPM) "
        "values.",
        "name": "get_rna_seq_archs4",
        "optional_parameters": [
            {
                "default": 10,
                "description": "The number of tissues to return",
                "name": "K",
                "type": "int",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "The gene name for which RNA-seq data is being fetched",
                "name": "gene_name",
                "type": "str",
            }
        ],
    },
    {
        "description": "Returns a list of supported databases for gene set enrichment analysis.",
        "name": "get_gene_set_enrichment_analysis_supported_database_list",
        "optional_parameters": [],
        "required_parameters": [],
    },
    {
        "description": "Perform enrichment analysis for a list of genes, with "
        "optional background gene set and plotting functionality.",
        "name": "gene_set_enrichment_analysis",
        "optional_parameters": [
            {
                "default": 10,
                "description": "Number of top pathways to return",
                "name": "top_k",
                "type": "int",
            },
            {
                "default": "ontology",
                "description": "Database to use for enrichment analysis (e.g., pathway, transcription, ontology)",
                "name": "database",
                "type": "str",
            },
            {
                "default": None,
                "description": "List of background genes to use for enrichment analysis",
                "name": "background_list",
                "type": "list",
            },
            {
                "default": False,
                "description": "Generate a bar plot of the top K enrichment results",
                "name": "plot",
                "type": "bool",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "List of gene symbols to analyze",
                "name": "genes",
                "type": "list",
            }
        ],
    },
    {
        "description": "Analyze chromatin interactions from Hi-C data to identify "
        "enhancer-promoter interactions and TADs.",
        "name": "analyze_chromatin_interactions",
        "optional_parameters": [
            {
                "default": "./output",
                "description": "Directory to save output files",
                "name": "output_dir",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to the Hi-C data file (.cool or .hic format)",
                "name": "hic_file_path",
                "type": "str",
            },
            {
                "default": None,
                "description": "Path to BED file containing genomic "
                "coordinates of regulatory elements "
                "(enhancers, promoters, CTCF sites, "
                "etc.)",
                "name": "regulatory_elements_bed",
                "type": "str",
            },
        ],
    },
    {
        "description": "Perform comparative genomics and haplotype analysis on "
        "multiple genome samples. Aligns genome samples to a "
        "reference, identifies variants, analyzes shared and unique "
        "genomic regions, and determines haplotype structure.",
        "name": "analyze_comparative_genomics_and_haplotypes",
        "optional_parameters": [
            {
                "default": "./output",
                "description": "Directory to store output files",
                "name": "output_dir",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Paths to FASTA files containing whole-genome sequences to be analyzed",
                "name": "sample_fasta_files",
                "type": "List[str]",
            },
            {
                "default": None,
                "description": "Path to the reference genome FASTA file",
                "name": "reference_genome_path",
                "type": "str",
            },
        ],
    },
    {
        "description": "Perform ChIP-seq peak calling using MACS2 to identify "
        "genomic regions with significant binding.",
        "name": "perform_chipseq_peak_calling_with_macs2",
        "optional_parameters": [
            {
                "default": "macs2_output",
                "description": "Prefix for output files",
                "name": "output_name",
                "type": "str",
            },
            {
                "default": "hs",
                "description": "Effective genome size shorthand: 'hs' for human, 'mm' for mouse, etc.",
                "name": "genome_size",
                "type": "str",
            },
            {
                "default": 0.05,
                "description": "q-value (minimum FDR) cutoff for peak calling",
                "name": "q_value",
                "type": "float",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to the ChIP-seq read data file (BAM, BED, or other supported format)",
                "name": "chip_seq_file",
                "type": "str",
            },
            {
                "default": None,
                "description": "Path to the control/input data file (BAM, BED, or other supported format)",
                "name": "control_file",
                "type": "str",
            },
        ],
    },
    {
        "description": "Find DNA sequence motifs enriched in genomic regions using the HOMER motif discovery software.",
        "name": "find_enriched_motifs_with_homer",
        "optional_parameters": [
            {
                "default": "hg38",
                "description": "Reference genome for sequence extraction",
                "name": "genome",
                "type": "str",
            },
            {
                "default": None,
                "description": "Path to BED file with background "
                "regions for comparison. If None, "
                "HOMER will generate random "
                "background sequences automatically",
                "name": "background_file",
                "type": "str",
            },
            {
                "default": "8,10,12",
                "description": "Comma-separated list of motif lengths to discover",
                "name": "motif_length",
                "type": "str",
            },
            {
                "default": "./homer_motifs",
                "description": "Directory to save output files",
                "name": "output_dir",
                "type": "str",
            },
            {
                "default": 10,
                "description": "Number of motifs to find",
                "name": "num_motifs",
                "type": "int",
            },
            {
                "default": 4,
                "description": "Number of CPU threads to use",
                "name": "threads",
                "type": "int",
            },
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "Path to peak file in BED format "
                "containing genomic regions to "
                "analyze for motif enrichment",
                "name": "peak_file",
                "type": "str",
            }
        ],
    },
    {
        "description": "Analyze overlaps between two or more sets of genomic regions.",
        "name": "analyze_genomic_region_overlap",
        "optional_parameters": [
            {
                "default": "overlap_analysis",
                "description": "Prefix for output files",
                "name": "output_prefix",
                "type": "str",
            }
        ],
        "required_parameters": [
            {
                "default": None,
                "description": "List of genomic region sets. Each "
                "item can be either a string path to "
                "a BED file or a list of "
                "tuples/lists with format (chrom, "
                "start, end) or (chrom, start, end, "
                "name)",
                "name": "region_sets",
                "type": "list",
            }
        ],
    },
]



import numpy as np
import pandas as pd

from biomni.task.base_task import base_task

np.random.seed(42)


def shuffle(x):
    np.random.shuffle(x)
    return x


class humanity_last_exam(base_task):
    def __init__(self, path="./data", category="Biology/Medicine", answer_type="multipleChoice"):
        if category not in [
            "Other",
            "Humanities/Social Science",
            "Math",
            "Physics",
            "Computer Science/AI",
            "Biology/Medicine",
            "Chemistry",
            "Engineering",
        ]:
            raise ValueError(
                "category must be one of ['Other', 'Humanities/Social Science', 'Math', 'Physics', 'Computer Science/AI', 'Biology/Medicine', 'Chemistry', 'Engineering']"
            )
        if answer_type not in ["exactMatch", "multipleChoice"]:
            raise ValueError("answer_type must be one of ['exactMatch' or 'multipleChoice']")

        self.dataset = category  # Store dataset type
        self.answer_type = answer_type
        df = pd.read_parquet(path + "/hle/test_sampled_biology_medicine.parquet")

        # Extract answer choices from question text
        def extract_options(question):
            # Find the "Answer Choices:" section
            if "Answer Choices:" not in question:
                return []
            choices = question.split("Answer Choices:")[1].strip()

            # Split on A., B., C. etc and clean up
            options = []
            letters = [
                "A.",
                "B.",
                "C.",
                "D.",
                "E.",
                "F.",
                "G.",
                "H.",
                "I.",
                "J.",
                "K.",
                "L.",
                "M.",
                "N.",
                "O.",
                "P.",
                "Q.",
                "R.",
                "S.",
                "T.",
                "U.",
                "V.",
                "W.",
                "X.",
                "Y.",
                "Z.",
            ]
            for i, letter in enumerate(letters):
                if letter in choices:
                    # Define the next letter if available
                    next_letter = letters[i + 1] if i + 1 < len(letters) else None

                    # Split between current letter and next letter
                    parts = choices.split(letter)[1]
                    if next_letter and next_letter in parts:
                        option = parts.split(next_letter)[0].strip()
                    else:
                        option = parts.strip()

                    options.append(option)
            return options

        def extract_question(question):
            return question.split("Answer Choices:")[0].strip()

        # Extract options and answers only for multiple choice questions and category is the same as the dataset
        df = df[df["category"] == self.dataset]
        df = df[df["answer_type"] == "multipleChoice"]
        df["question_text"] = df.question
        df["letter_answer"] = df["answer"].apply(lambda x: x[0])

        self.query = df.question_text.values
        # self.options = df.options_letters.values
        self.answer = df.letter_answer.values

        self.prompt = """Question: {question}"""

    def get_example(self, index=None):
        if index is None:
            index = np.random.randint(len(self.query))

        return {
            "prompt": self.prompt.format(
                question=self.query[index],
                # options = self.options[index]
            ),
            "answer": self.answer[index],
        }

    def get_iterator(self):
        for i in range(len(self.query)):
            yield self.get_example(i)

    def evaluate(self, response):
        ## expected a list/array of symbols
        from sklearn.metrics import accuracy_score

        ground_truth = self.answer
        response = np.array(response)

        return {
            "accuracy": accuracy_score(ground_truth, response),
            "coverage": np.mean(response != self.refrain_label),
            "refrain_ratio": np.mean(response == self.refrain_label),
            "precision": accuracy_score(
                ground_truth[np.where(response != self.refrain_label)],
                response[np.where(response != self.refrain_label)],
            ),
        }

    def output_class(self):
        from typing import Optional

        from pydantic import BaseModel, Field

        class MultipleChoiceOutput(BaseModel):
            """Multiple choice output."""

            choice: str | None = Field(
                description="Multiple choice answer. For example, if there is <answer>A</answer> in the prompt, the output should be 'A'."
            )

        return MultipleChoiceOutput
